#!/usr/bin/python
# -*- coding: utf-8 -*-

import json
from os.path import abspath, join, isfile, isdir
from os import getcwd, remove, rename as filerename
import random
from mpd import MPDClient

class iRadio_Exception(Exception):
    pass

class iRadio_SaveError(iRadio_Exception):
    pass

class iRadio():
    currentStationUrl = ""
    currentStationName = ""
    currentStation = None
    currentVolume = 60
    currenSuspended = False
    jsonFilename = ''
    jsonChanged = False
    
    radio  = None
    radios = None

    mpdClient = None
    
    def __init__(self, radioList="radio"):
        self.load(radioList)
        self.mpdClient = MPDClient()               # create client object
        self.mpdClient.timeout = 20                # network timeout in seconds (floats allowed), default: None
        self.mpdClient.idletimeout = None          # timeout for fetching the result of the idle command is handled seperately, default: None


    def __del__(self):
        if self.jsonChanged: 
            self.save(msg="{} saved because changed")


    def load(self, name):
        name = name + '.json'
        self.jsonFilename = name
        filename = join(abspath(getcwd()), name)
        self.radio = json.loads( open(filename).read() )
        self.radios = self.radio["radio"]
        self.jsonChanged = False


    def save(self, name="", msg=None):
        if not self.radio: return
        if name == "":
            name = self.jsonFilename
        else:
            name = name + '.json'
        if (name == '') or (name == ".json"):
            raise AppExc_SaveError('No file name specified')

        name = join(abspath(getcwd()), name)
        backFilename = name.replace('.json', '.bak')
        if isdir(backFilename):
            print('could not backup "{0}", a directory named "{1}" already exists'.format(name, backFilename))
        else:
            if isfile(backFilename):
                remove(backFilename)
                #print("removed ", backFilename)
            if isfile(name):
                filerename(name, backFilename)
            #print('"{0}" backed up to \n"{1}"'.format(name, backFilename))
        with open(name, 'w') as outfile:
            json.dump(self.radio, outfile, indent=4, sort_keys=False, separators=(',', ': '), ensure_ascii=False)
        if msg: print(msg.format(name))
        self.jsonChanged = False


    def findSimilar(self, name):
        sourceWords = {}
        sourceLen = 0

        def intersection(stationName):
            stationWords = set(stationName.lower().split())
            stationLen = len(stationWords)
            if stationLen == 0:
                return 10000, 10000
            common = len(sourceWords & stationWords)
            if common == 0:
                return 10000, 10000
            return sourceLen - common, stationLen - common

        sourceWords = set(name.lower().split())
        sourceLen = len(sourceWords)

        if sourceLen < 1:
            return None, 10000, 10000

        leftSource = None
        leftInCount = 0
        leftOutCount = 0

        rightSource = None
        rightInCount = 0
        rightOutCount = 0

        for station in self.radios:
            inCount, outCount = intersection(station["name"])
            if (inCount == 0) and (outCount == 0):
                return station, 0, 0
            if (not leftSource) or (inCount < leftInCount):
                leftSource = station
                leftInCount = inCount
                leftOutCount = outCount
            if (not rightSource) or (outCount < rightOutCount):
                rightSource = station
                rightInCount = inCount
                rightOutCount = outCount

        if leftInCount == 0:
            return leftSource, leftInCount, rightOutCount
        elif rightInCount == 0:
            return rightSource, rightInCount, rightOutCount
        elif (leftInCount >= 10000) and (rightInCount >= 10000):
            return None, 10000, 10000
        elif (leftInCount + leftOutCount < rightInCount + rightOutCount):
            return leftSource, leftInCount, rightOutCount
        else:
            return rightSource, rightInCount, rightOutCount


    def mpdConnect(self):
        self.mpdClient.connect("localhost", 6600)  # connect to localhost:6600
        
    
    def mpdDisconnect(self):
        self.mpdClient.close()
        self.mpdClient.disconnect()


    def status(self):
        self.mpdConnect()
        print(self.mpdClient.status())
        self.mpdDisconnect()
        
        
    def play(self, station):

        #print("station: ", station)
        
        if isinstance(station, str):
            station, inc, outc = self.findSimilar(station)

        if not station:
            return False, "unknown source" 

        self.mpdConnect()

        playing = False
        srcs = station["src"]
        ## possible errors here if empty or station is not
        ## the correct object
        ## insert call back   'Searching for {} on the Internet'.station["name"]
        for i, url in enumerate(srcs):
            self.mpdClient.clear()
            self.mpdClient.add(url)
            self.mpdClient.play(0)
            status = self.mpdClient.status()
            if "error" in status:
                #print("{} is not a valid url".format(url))
                continue
            else:
                playing = True
                #print("{} is a valid url".format(url))
                #print("i = ", i)
                if i > 0: 
                    #print("inserting url ", i, " at 0")
                    srcs.insert(0, srcs.pop(i))
                    self.jsonChanged = True
                self.currentStation = station
                self.currentStationName = station['name']
                self.currentStationUrl = url
                break
        
        if playing:
            print('Playing {}'.format(station['name']))
            print('\tURL = {}'.format(self.currentStationUrl))
        else:
            self.mpdClient.stop()
            self.mpdClient.clear()
            self.currentStation = None
            self.currentStationName = ""
            self.currentStationUrl = ""
            print('No valid URL to play {}'.format(station['name']))

        self.mpdDisconnect()
        self.currentSuspended = False
        return playing, station["name"]


    def setVolume(self, volume):
        if volume and (volume != self.currentVolume) and self.currentStation:
            self.mpdConnect()
            self.mpdClient.setvol(volume)
            self.currentVolume = volume
            self.mpdDisconnect()
        return self.currentVolume


    def listStations(self, region="", genre=""):
        rlist = []
        for station in self.radios:
            if (region == "") or (region == station["region"]):
                if (genre == "") or (genre in station["cat"]):
                    rlist.append(station["name"])
        return rlist


    def pause(self):
        if self.currentStation and not self.currentSuspended:
            self.mpdConnect()
            self.mpdClient.pause()    
            self.mpdDisconnect()         
            self.currentSuspended = True
        

    def resume(self):
        if self.currentStation and self.currentSuspended:
            self.mpdConnect()
            self.mpdClient.pause()    
            self.mpdDisconnect()         
            self.CurrentSuspended = False
   
                            
    def stop(self):
        self.mpdConnect()
        if self.currentStation:
            name = self.currentStationName
        else:
            name = self.mpdClient.currentsong()
        self.mpdClient.clear()
        self.mpdClient.stop()
        self.mpdDisconnect()
        if name:
            print('Stopped playing {}'.format(name))
        self.currentStation = None
        self.currentStationName = ''
        self.currentStationUrl = ''


    def playRandom(self, region="", genre=""):
        alist = self.listStations(region, genre)
        print(alist)
        if len(alist) == 0:
            return False, "No station found"
        else:
            while len(alist) > 0:
                astation = random.choice(alist)
                ok, msg = self.play(astation)
                if ok:
                    return ok, msg
                else:
                    alist.remove(astion)
            return False, "No station found"        
                

def _main():

    import argparse
    import time
    import picotts

    parser = argparse.ArgumentParser(description='Test iradio script')
    parser.add_argument('station', nargs='*', help='Name of station to play')
    parser.add_argument('-v', '--volume', type=int, help='set the volume (1-100)')
    parser.add_argument('-s', '--stop', action="store_true", help='stop playing')
    parser.add_argument('-p', '--pause', action="store_true", help='test pause')
    parser.add_argument('-t', '--tts', action="store_true", help='test with picoTTS')
    parser.add_argument('-m', '--mpd', action="store_true", help='show mpd status')
        
    args = parser.parse_args()

    radio = iRadio()
    
    if args.station:
        radio.play(' '.join(args.station))
        
    if args.volume:
        radio.setVolume(args.volume)
  
    if args.stop:
        radio.stop()
        
    if args.mpd:
        radio.status()
    
    if args.pause:
        if not args.station:
            radio.playRandom()
        time.sleep(100) 
        print('pausing for 10 seconds')
        radio.pause()
        time.sleep(10)
        radio.resume()
        print('resumed')
        
    
    if args.tts:
        if not args.station:
            radio.playRandom()
        radio.setVolume(90)

        time.sleep(10) 
        print('test of picotts with pause')
        radio.pause()
        time.sleep(5)
        picotts.say('This is a test, using picotts to say a message while the music player daemon is paused', lang="en-US")
        time.sleep(2)
        radio.resume()

        time.sleep(10)
        print('test of picotts concurrently with mpd')
        radio.setVolume(30)
        time.sleep(5)
        picotts.say('This is a test, using picotts to say a message while the music player daemon is set to a low volume', lang="en-US", volume=90)
        time.sleep(2)
        radio.setVolume(90)


if __name__ == '__main__':
    _main()


