#!/usr/bin/env python
#-*- coding: utf-8 -*-

# Python wrapper for Sox pico tts engine 

import subprocess
import os
import tempfile
from configuration import Configuration

defaultLang = 'fr-FR'
defaultVolume = 120
defaultPitch = 100
defaultSpeed = 100
echoWords = True

LANGS = [ 'de-DE', 'en-GB', 'en-US', 'es-ES', 'fr-FR', 'it-IT' ]
wavfile = ''


def say(words, lang=None, volume=None, pitch=None, speed=None):
    if not lang: lang = defaultLang
    if not volume: volume = defaultVolume
    if not pitch: pitch = defaultPitch
    if not speed: speed = defaultSpeed
    #print('lang: "{0}", volume: "{1}", pitch: "{2}", speed: "{3}"'.format(lang, volume, pitch, speed))

    if echoWords: print(words)
    words = '<volume level="{}"><pitch level="{}"><speed level="{}">{}</speed></pitch></volume>'.format(
        volume, pitch, speed, words)
    try:
        with open("/dev/null","w") as devnull:
            subprocess.call(["pico2wave", "-l", lang, "-w", wavfile, words], stderr=devnull)
            subprocess.call(["aplay", wavfile],stderr=devnull)
    finally:
        os.unlink(wavfile)


def setDefaultLang(lang):
    global defaultLang
    if lang in LANGS:
        defaultLang = lang
    else:
        print("'{0}' is not among the supported languages: {1}".format(lang, LANGS))


def setDefaultVolume(vol):
    global defaultVolume
    if 0 <= vol <= 500:
        defaultVolume = vol
    else:
        print("'{0}' is a volume outside the valid range: [0 - 500]".format(vol))


def setDefaultPitch(pitch):
    global defaultPitch
    if 50 <= pitch <= 200:
        defaultPitch = pitch
    else:
        print("'{0}' is a pitch outside the valid range: [50..200]".format(pitch))


def setDefaultSpeed(speed):
    global defaultPitch
    if 30 <= speed <= 500:
        defaultSpeed = speed
    else:
        print("'{0}' is a speed outside the valid range: [30..500]".format(vol))


def loadConfig():
    configuration = Configuration()
    config = configuration.loadConfig()["tts"]
    setDefaultLang(config["lang"])
    setDefaultVolume(config["volume"])
    setDefaultPitch(config["pitch"])
    setDefaultSpeed(config["speed"])
    echoWords = config["echo_words"]
    

def _main():
    
    import argparse

    parser = argparse.ArgumentParser(description='Test pico2wave wrapper')
    parser.add_argument('words', nargs='*', help='Words to say')
    parser.add_argument('-l', '--lang', type=str, help='Output language')
    parser.add_argument('-v', '--vol', type=int, help='Output volume')
    parser.add_argument('-p', '--pitch', type=int, help='Output pitch')
    parser.add_argument('-s', '--speed', type=int, help='Output speed')
    parser.add_argument('-d', '--defaults', help="Show defaults", action="store_true")
    
    args = parser.parse_args()

    #print("words: ", args.words)
    #print("--lang: ", args.lang)
    #print("--vol: ", args.vol)
    #print("--pitch: ", args.pitch)
    #print("--speed: ", args.speed)
    
    if args.defaults:
        print("Default settings\n\tdefaultLang: {}".format(defaultLang))
        print("\tdefaultVolume: {}\n\tdefaultPitch {}".format(defaultVolume, defaultPitch))
        print("\tdefaultSpeed: {}".format(defaultSpeed))
        
    if args.words:
        words = ' '.join(args.words)
        say(words, args.lang, args.vol, args.pitch, args.speed)


try:
    (fd, wavfile) = tempfile.mkstemp(suffix='.wav', dir='/run/user/%d' % os.getuid())
except IOError:
    #logger.exception('Using fallback directory for TTS output')
    (fd, wavfile) = tempfile.mkstemp(suffix='.wav')
    os.close(fd)
#print('wavfile: "{}"'.format(wavfile))
#os.system('cat /etc/mtab | grep /run/user/%d' % os.getuid())


loadConfig()


if __name__ == '__main__':
    _main()
